package com.sample;

import java.io.File;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.builder.conf.DeclarativeAgendaOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;



/**
 * 
 * Possible declarative agenda bug in cancelling activation.
 *
 * @author jsvitak@redhat.com
 * @version BRMS 5.3.0 DEV5
 *
 */
public class DeclarativeAgendaCancel {
    
    public static void main(String args[]) {
            KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
            kbuilder.add(ResourceFactory.newFileResource(new File("test-resources", "declarative-agenda-cancel.drl")), ResourceType.DRL);
            
            //System.out.println(kbuilder.getErrors().toString());

            KnowledgeBaseConfiguration kconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
            kconf.setOption(DeclarativeAgendaOption.ENABLED);
            KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(kconf);
            kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

            StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
           
            // first run
            // sales1 should be cancelled
            System.out.println("#### first run - sales1 is cancelled, everything works fine");
            FactHandle fireRules = ksession.insert("fireRules");
            FactHandle fireCancelRule = ksession.insert("fireCancelRule");
            ksession.fireAllRules();

            // second run
            // sales1 should be cancelled again, but it isn't
            // expected behaviour should be the same as in the first run
            System.out.println("\n#### second run - updated both facts, expected behaviour should be the same as in the first run");
            ksession.update(fireCancelRule, "fireCancelRule");
            ksession.update(fireRules, "fireRules");
            ksession.fireAllRules();

            ksession.dispose();
    }

}
